unit DDUtil440;

interface

uses
    Windows, DDDD ,DDraw, Classes, SysUtils;

type
    TSurfaceList = class
      private
        FDD:TDDDD;
        FList:TStringList;
        function    Add(S:String):TDDDDSurface;
        function    DeleteHonki(Index:Integer):Boolean;
        function    IndexOf(Surface:TDDDDSurface):Integer;
      protected
      public
        constructor Create(ADD:TDDDD);
        destructor  Destroy;override;

        procedure   ReadPalette(const FileName:String);

        procedure   ClearAll;
        function    LoadBMP(const FileName:String):TDDDDSurface;
        function    Delete(S:String):Boolean;overload;
        function    Delete(Index:Integer):Boolean;overload;
        function    Delete(Surface:TDDDDSurface):Boolean;overload;
        function    LegalIndex(Index:Integer):Boolean;
        
        procedure   ReadFromStringArray(S:Array of String);
        procedure   ReadFromStrings(S:TStrings);
        procedure   ReadFromListFile(FileName:String);
    end;

    procedure ReadBMPHeader(const FileName:String; var bi:TBitmapInfoHeader);

implementation

procedure ReadBMPHeader(const FileName:String; var bi:TBitmapInfoHeader);
var
  f:File;
begin

     if not FileExists(FileName) then exit;

     AssignFile(f,FileName);
     Reset(f,1);
     Seek(f,sizeof(TBitmapFileHeader));
     BlockRead(f,bi,sizeof(TBitmapInfoHeader));
     CloseFile(f);
end;

{ TSurfaceList }

//ۂɃIuWFNg𐶐A
//t@Cǂݍފ֐
function TSurfaceList.Add(S: String): TDDDDSurface;
var
   tmp:TDDDDSurface;
   bi:TBitmapInfoHeader;
begin

     ReadBmpHeader(S,bi);
     tmp:=TDDDDSurface.Create(FDD,bi.biWidth,bi.biHeight,0);
     tmp.LoadBMP(S);

     result:=tmp;

end;

//Xg̓eNAB
procedure TSurfaceList.ClearAll;
begin
     while FList.Count > 0 do
     begin
          DeleteHonki(0);
     end;
end;

constructor TSurfaceList.Create(ADD: TDDDD);
begin
     FList:=TStringList.Create;
     FDD:=ADD;
end;

function TSurfaceList.Delete(S: String): Boolean;
begin
     result:=Delete(FList.IndexOf(S));
end;

function TSurfaceList.Delete(Index: Integer): Boolean;
begin
     if LegalIndex(Index) then
     begin
          result:=DeleteHonki(Index);
     end

     else
     begin
          result:=False;
     end;
end;

function TSurfaceList.Delete(Surface: TDDDDSurface): Boolean;
begin
     result:=Delete(IndexOf(Surface));
end;

function TSurfaceList.DeleteHonki(Index: Integer): Boolean;
begin
     (FList.Objects[Index] as TDDDDSurface).Free;
     FList.Delete(Index);

     result:=True;
end;

destructor TSurfaceList.Destroy;
begin
     ClearAll;
     FList.Free;
     inherited;
end;

function TSurfaceList.IndexOf(Surface: TDDDDSurface): Integer;
var
   i:Integer;
   tmp:TObject;
begin
     result:=-1;
     for i:= 0 to FList.Count-1 do
     begin
          tmp:=FList.Objects[i];
          if Surface = tmp then
          begin
               result:=i;
               exit;
          end;
     end;
end;

//^ꂽIndexlz̐vf
//wĂ邩𒲂ׂ֐
function TSurfaceList.LegalIndex(Index: Integer): Boolean;
begin
     result:=(Index >= 0) and (Index < FList.Count);
end;

//t@C1ǂݍފ֐
//ǂݍݍς݂Ȃ炻Ԃ
//ǂݍłȂȂǂݍł炻Ԃ
//t@Cnil(NULL)Ԃ
function TSurfaceList.LoadBMP(const FileName: String): TDDDDSurface;
var
   Index:Integer;
begin
     result:=nil;

     //t@C̗L`FbN
     if not FileExists(FileName) then Exit;

     //Xgɂłɂ肷񂿂ႤH
     Index:=FList.IndexOf(FileName);
     if LegalIndex(Index) then
     begin
          result:=FList.Objects[Index] as TDDDDSurface;
          exit;
     end;

     //ǂݍ
     FList.AddObject(FileName,Add(FileName));
     result:=FList.Objects[FList.Count-1] as TDDDDSurface;
end;

//ǂݍ݂t@C̃XgeLXg
//t@CCɓǂݍ
procedure TSurfaceList.ReadFromListFile(FileName: String);
var
   TempList:TStringList;
begin
     TempList:=TStringList.Create;
     //Xgt@Cǂݍ
     if FileExists(FileName) then
     begin
          TempList.LoadFromFile(FileName);
          ReadFromStrings(TempList);
     end;
     TempList.Free;
end;

//ǂݍ݂t@CzƂēn
//CɑSǂݍł
procedure TSurfaceList.ReadFromStringArray(S: array of String);
var
   i:Integer;
begin
     for i:=Low(S) to High(S) do
     begin
          LoadBmp(S[i]);
     end;
end;

//ǂݍ݂t@CTStringsɓēn
//CɑSǂݍł
procedure TSurfaceList.ReadFromStrings(S: TStrings);
var
   i:Integer;
begin
     for i:= 0 to S.Count-1 do begin
         LoadBmp(S[i]);
     end;
end;

procedure TSurfaceList.ReadPalette(const FileName: String);
begin
     if not FileExists(FileName) then exit;

     FDD.ReadPalette(FileName);
     FDD.UpdatePalette;
end;

end.

