unit MathLib440;

interface

uses
    Windows,SysUtils;

//Op֐n
const
     ARC_ANGLE_MAX  = 4096;
     ARC_ANGLE_MASK = ARC_ANGLE_MAX-1;

     SIN_TERM = 4096;
     SIN_MASK = SIN_TERM-1;

function Atan2(x,y : SmallInt) : SmallInt;
function SinFast(Angle: Integer): Single;
function CosFast(Angle: Integer): Single;

var
   ATan2Table:array [0..ARC_ANGLE_MAX-1] of Integer;
   SinTable    :array [0..SIN_TERM-1] of Single;

//Œ菬_n
const
     FixedDecimalBit   = 32;
     FixedDecimalShift = FixedDecimalBit div 2;

     FixedDecimalDiv  = 1 shl (FixedDecimalShift      );
     FixedDecimalDiv2 = 1 shl (FixedDecimalShift div 2);
     FixedDecimalDiv4 = 1 shl (FixedDecimalShift div 4);

     FixedMultiplier: Real = FixedDecimalDiv * 1.0;

type
    TFixedDecimal = record
      case Boolean of
        True :(Value:LongInt);
        False:(Frac:Word;Int:SmallInt);
    end;

function IntToFixedDecimal(Value:Integer):TFixedDecimal;
function SingleToFixedDecimal(Value:Single):TFixedDecimal;
function DoubleToFixedDecimal(Value:Double):TFixedDecimal;

function FixedDecimalToInt(Value:TFixedDecimal):Integer;
function FixedDecimalToSingle(Value:TFixedDecimal):Single;
function FixedDecimalToDouble(Value:TFixedDecimal):Double;

implementation

//Op֐n
procedure InitSankakuTable;
var
   i,j,k,a:Integer;
   angle:double;
begin
     i := 0;
     for j := 0 to ARC_ANGLE_MAX shr 3 do
     begin
          angle:=j/ARC_ANGLE_MAX*2*PI;
          a := Round(Sin(angle) / Cos(angle) * ARC_ANGLE_MAX);
          for k := 0 to a - i do
          begin
               Atan2Table[i+k] := j;
          end;
          i := a;
     end;
     for k := 0 to ARC_ANGLE_MAX - i do
     begin
          Atan2Table[i+k] := ARC_ANGLE_MAX shr 3;
     end;

     for i:= 0 to SIN_TERM-1 do
     begin
          SinTable[i]:=Sin(i/SIN_TERM*2*PI);
     end;

end;

function  Atan2(x,y : SmallInt) : SmallInt;
var
   res : SmallInt;
begin
     if ((x = 0) and (y = 0))then
     begin
          result := 0;
          Exit;
     end;

     if (x > 0) then
        if (y > 0) then
           if (x > y) then res :=         Atan2Table[y shl 12 div x]
		      else res := 256*4 - Atan2Table[x shl 12 div y]
        else
        begin
             y := -y;
             if (x > y) then res := 1024*4 - Atan2Table[y shl 12 div x]
                        else res := 768*4  + Atan2Table[x shl 12 div y];
        end

     else
     if (y > 0)then
     begin
          x := -x;
          if (x > y) then res := 512*4 - Atan2Table[y shl 12 div x]
		     else res := 256*4 + Atan2Table[x shl 12 div y];
     end
     else
     begin
          x := -x;
          y := -y;
          if (x > y) then res := 512*4 + Atan2Table[y shl 12 div x]
		     else res := 768*4 - Atan2Table[x shl 12 div y];
          end;

  result := res;

end;

function CosFast(Angle: Integer): Single;
begin
     result:=SinTable[(Angle+SIN_TERM shr 2) and SIN_MASK];
end;

function SinFast(Angle: Integer): Single;
begin
     result:=SinTable[Angle and SIN_MASK];
end;

//Œ菬_n
function IntToFixedDecimal(Value:Integer):TFixedDecimal;
begin
     Result.Int  := Value;
     Result.Frac := 0;
end;

function SingleToFixedDecimal(Value:Single):TFixedDecimal;
begin
     Result.Value := Round(Value * FixedMultiplier);
end;

function DoubleToFixedDecimal(Value:Double):TFixedDecimal;
begin
     Result.Value := Round(Value * FixedMultiplier);
end;

function FixedDecimalToInt(Value:TFixedDecimal):Integer;
begin
    Result := Value.Int;
    if FixedDecimalDiv <= Value.Frac div 2 then Inc(Result);
end;

function FixedDecimalToSingle(Value:TFixedDecimal):Single;
begin
     Result := Value.Value / FixedMultiplier;
end;

function FixedDecimalToDouble(Value:TFixedDecimal):Double;
begin
     Result := Value.Value / FixedMultiplier;
end;

initialization
  InitSankakuTable;

end.
