unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  dddd, ddraw, DDUtil440, AppLib440, MathLib440, Ddidex, IniFiles;

//{$DEFINE BOUND}
{$DEFINE USE_BLUR}
//{$DEFINE POINT_SPRITE}
//{$DEFINE USE_FIXEDDECIMAL}

const

     MAX_HEIGHT = 2048;

     PARTICLE_SIZE   =    2;
     TRANSPARENT_COLOR = 0;

     FLAG_WAIT = 0;
     FLAG_MOVE = 1;

     RAND_SIZE = 65536;
     RAND_SIZE_MASK = RAND_SIZE-1;

type
  ppByte = ^pByte;

  PParticle = ^TParticle;
  TParticle = record
    {$IFDEF USE_FIXEDDECIMAL}
    x,y  :LongInt;
    vx,vy:LongInt;
    mp   :LongInt;
    {$ELSE}
    x,y  :double;
    vx,vy:double;
    mp   :double;
    {$ENDIF}
    t    :Integer;
    deadt:Integer;
    flag :Integer;
    draw :Boolean;
  end;

  TMainForm = class(TForm)
    DD: TDDDD;
    DI: TDDIDEX;
    procedure DDInit(Sender: TObject; fNeedChange: Boolean);
    procedure DDDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private 錾 }
    //ϐ
    {$IFDEF USE_FIXEDDECIMAL}
     GRAVITY:LongInt;
     M      :LongInt;
     GENSUI :LongInt;
     BOUND  :LongInt;
     FV     :LongInt;
     {$ELSE}
     GRAVITY:double;
     M      :double;
     GENSUI :double;
     BOUND  :double;
     FV     :double;
     {$ENDIF}

     MAX_PARTICLE    : Integer;
     PARTICLE_COLOR  : Byte;
     PARTICLE_ENERGY : Integer;

     SCREEN_WIDTH    : Integer;
     SCREEN_HEIGHT   : Integer;
     ANGLE_RANGE     : Integer;

     FIniFileName:String;
     FStrong:Integer;
     FAngle:Integer;
     FMouseX:Integer;
     FMouseY:Integer;
     FMouseDowned:Boolean;

     FCenterX:Integer;
     FCenterY:Integer;

     FDeadCount:Integer;
     FSpeed:Integer;
     FFps:Integer;
     FFpsTemp:Integer;
     FFpsTime:Integer;

     FRndStep :Integer;
     FRndIntStep :Integer;

     FTime    :Integer;
     FSurface :TDDDDSurface;
     FBlur    :TDDDDSurface;
     FMouse   :TDDDDSurface;

    FAddTable:array [0..256*2-1] of Byte;
    //FWater   :array [0..MAX_PARTICLE-1] of TParticle;
    FOffsetTable :array [0..MAX_HEIGHT-1] of pByte;
    FWater   :PParticle;
    //FOffsetTable :ppByte;
    FDivTable:array [0..256*5-1] of Integer;
    FRandTable   :array [0..RAND_SIZE-1] of Single;
    FRandIntTable:array [0..RAND_SIZE-1] of Integer;
    function  RandomFast:Single;
    function  RandomIntFast:Integer;
  public
    { Public 錾 }
    FSurfaceList:TSurfaceList;
    procedure LoadIni;
    procedure SaveIni;
    procedure UpdateMouse;
    procedure Fps;
    procedure Blur;
    procedure InitTable;
    procedure InitWater;
    procedure InitMouse;
    procedure InitOffsetTable;
    procedure FreeWater;
    procedure FreeOffsetTable;
    procedure MoveWater(dt: integer);
    procedure StartParticle(Index:Integer);
    procedure EndParticle(Index:Integer);
    procedure MainLoop(Sender:TObject;var Done:Boolean);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

{ TForm1 }

procedure TMainForm.MainLoop(Sender: TObject; var Done: Boolean);
var
   DDCanvas:TDDDDCanvas; 
begin
     Done:=False;

     Fps;

     UpdateMouse;

     MoveWater(FSpeed);
     FSurface.BltFast(FMouse,FMouseX,FMouseY,Rect(0,0,FMouse.Width,FMouse.Height),
                      DDBLTFAST_WAIT or DDBLTFAST_SRCCOLORKEY);
     Blur;
     FBlur.PutFast(1,1,Rect(1,1,FSurface.Width-1,FSurface.Height-1),DDBLTFAST_WAIT);
     FBlur   .FillRect(Rect(0,0,FBlur.Width,FBlur.Height),TRANSPARENT_COLOR);

     DDCanvas:=TDDDDCanvas.Create(DD.BackBuffer);
     if DDCanvas.Enabled then
     begin
          DDCanvas.TextOut(0,  0,Format('%d FPS' + ' ( ESCŏIł܂ )',[FFps]));
          //DDCanvas.TextOut(0, 12,Format('DeadCount:%d',[FDeadCount]));
          //DDCanvas.TextOut(0, 24,Format('x:%d',[FMouseX]));
          //DDCanvas.TextOut(0, 36,Format('y:%d',[FMouseY]));
     end;
     DDCanvas.Free;

     DD.Flip;
end;

procedure TMainForm.DDInit(Sender: TObject; fNeedChange: Boolean);
begin

     FSurfaceList:=TSurfaceList.Create(DD);
     FSurfaceList.ReadPalette(ExePath+'logo.bmp');

     FSurface :=TDDDDSurface.Create(DD,SCREEN_WIDTH,SCREEN_HEIGHT,TRANSPARENT_COLOR);
     FSurface .FillRect(Rect(0,0,FSurface.Width,FSurface.Height),TRANSPARENT_COLOR);

     FBlur    :=TDDDDSurface.Create(DD,SCREEN_WIDTH,SCREEN_HEIGHT,TRANSPARENT_COLOR);
     FBlur    .FillRect(Rect(0,0,FBlur.Width,FBlur.Height),TRANSPARENT_COLOR);

     FMouse:=FSurfaceList.LoadBMP(ExePath+'logo.bmp');

     InitOffsetTable;

     ClientWidth :=SCREEN_WIDTH;
     ClientHeight:=SCREEN_HEIGHT;

     DD.Clear;
     DD.Flip;
     DD.Clear;
end;

procedure TMainForm.DDDestroy(Sender: TObject);
begin
     FSurfaceList.Free;

     FSurface.Free;
     FBlur.Free;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
     FIniFileName:=ChangeFileExt(ParamStr(0),'.ini');
     LoadIni;
     FSpeed:=64;
     InitMouse;
     InitTable;
     InitWater;
     Application.OnIdle:=MainLoop;
     HideMouseCursor;

     //DD.WindowMode(SCREEN_WIDTH,SCREEN_HEIGHT);
     DD.FullScreenMode(SCREEN_WIDTH,SCREEN_HEIGHT,8,0,'');
end;

procedure TMainForm.InitWater;
var
   i:Integer;
   Particle:PParticle;
begin
     FTime:=0;

     GetMem(FWater,MAX_PARTICLE*SizeOf(TParticle));

     Particle:=FWater;
     for i:= 0 to MAX_PARTICLE-1 do
     begin
          with Particle^ do
          begin
               flag:=Random(256) and 1;
               if flag = FLAG_MOVE then
               begin
                    StartParticle(i);
               end else
               begin
                    EndParticle(i);
               end;
          end;
          Inc(Particle);
     end;

end;

procedure TMainForm.MoveWater(dt: Integer);
var
   i,dx,dy:Integer;
   dtl:double;
   Particle:PParticle;
   {$IFDEF POINT_SPRITE}
   ddsd:DDSurfaceDesc2;
   p:pByte;
   {$ENDIF}
begin
     FDeadCount:=0;
     {$IFDEF POINT_SPRITE}
     FSurface.Lock(ddsd);
     {$ENDIF}

     dtl:=dt / 1000;
     Inc(FTime,dt);

     Particle:=FWater;
     for i:= 0 to MAX_PARTICLE-1 do
     begin
          if Particle^.flag = FLAG_WAIT then
          begin
               Inc(FDeadCount);
               with Particle^ do
               begin
                    Dec(deadt,dt);
                    if deadt < 0 then StartParticle(i);
               end;
          end else
          begin
               with Particle^ do
               begin
                    Inc(t,dt);

                    Dec(deadt,dt);
                    if deadt < 0 then EndParticle(i);

                    {$IFDEF USE_FIXEDDECIMAL}

                    //Inc(vy,GRAVITY * mp * dt div 1024 div 65536);
                    //vy.Value:=vy.Value+((GRAVITY*mp.Value) div FixedDecimalDiv2 * dt) div FixedDecimalDiv2;

                    //x.Value:=x.Value+(vx.Value*dt) div FixedDecimalDiv2;
                    //y.Value:=y.Value+(vy.Value*dt) div FixedDecimalDiv2;
                    Inc(vy,GRAVITY * 3 * dt div 1024);

                    Inc(x,vx * dt div 1024);
                    Inc(y,vy * dt div 1024);

                    dx:=x div 65536;
                    dy:=y div 65536;
                    {$ELSE}
                    vy:=vy+GRAVITY*mp*dtl;

                    x:=x+vx*dtl;
                    y:=y+vy*dtl;

                    dx:=Round(x);
                    dy:=Round(y);
                    {$ENDIF}

                    draw:=(dx>=0)and(dx<SCREEN_WIDTH)and
                          (dy>=0)and(dy<SCREEN_HEIGHT);

                    if draw then
                    begin
                         {$IFDEF POINT_SPRITE}
                         p:=FOffsetTable[dy];
                         Inc(p,dx);
                         p^:=PARTICLE_COLOR;//FAddTable[p^+PARTICLE_COLOR];
                         {$ELSE}
                         FSurface.FillRect(Bounds(dx,dy,2,2),PARTICLE_COLOR);
                         {$ENDIF}
                    end;
               end;
          end;
          Inc(Particle);
     end;
     {$IFDEF POINT_SPRITE}
     FSurface.UnLock;
     {$ENDIF}
end;

procedure TMainForm.StartParticle(Index: Integer);
var
   Angle:Integer;
   Particle:PParticle;
   temp:TParticle;
begin
     Particle:=FWater;
     Inc(Particle,Index);
     with Particle^ do
     begin
          t:=0;
          Angle:=FAngle-ANGLE_RANGE shr 1+Random(ANGLE_RANGE);
          {$IFDEF USE_FIXEDDECIMAL}
                  x :=(FCenterX-8+Random(16))*65536;
                  y :=(FCenterY-8+Random(16))*65536;
                  vx:=Round( FStrong*CosFast(Angle)*65536);
                  vy:=Round(-FStrong*SinFast(Angle)*65536);
                  mp:=Round(M+RandomFast*65536);
          {$ELSE}
                 x :=FCenterX-8+Random(16);
                 y :=FCenterY-8+Random(16);
                 vx:= FStrong*CosFast(Angle);
                 vy:=-FStrong*SinFast(Angle);
                 mp:=M+RandomFast;
          {$ENDIF}
          temp:=Particle^;
          deadt:=PARTICLE_ENERGY+RandomIntFast;
          flag:=FLAG_MOVE;
     end;
end;

procedure TMainForm.EndParticle(Index: Integer);
var
   Particle:PParticle;
begin
     Particle:=FWater;
     Inc(Particle,Index);
     with Particle^ do
     begin
          t:=0;
          deadt:=0;
          flag:=FLAG_WAIT;
     end;
end;

procedure TMainForm.Blur;
{$IFDEF USE_BLUR}
var
   x,y:Integer;
   ddsd,dddsd:DDSurfaceDesc2;
   p1,p2,p3,p4,p5,p6:pByte;
{$ENDIF}
begin
{$IFDEF USE_BLUR}
     //ڂ
     FSurface.Lock(ddsd);
     FBlur   .Lock(dddsd);

     p1:=ddsd.lpSurface;   //p1=[x-1,y]
     Inc(p1,ddsd.lPitch);
     p2:=p1;               //p2=[x+1,y]
     Inc(p2,2);

     p3:=ddsd.lpSurface;   //p3=[x,y-1]
     Inc(p3);
     p4:=p3;               //p4=[x,y+1]
     Inc(p4,2*ddsd.lPitch);

     p5:=ddsd.lpSurface;   //p5=[x,y]
     Inc(p5);
     Inc(p5,ddsd.lPitch);

     p6:=dddsd.lpSurface;  //p6=[x,y]
     Inc(p6);
     Inc(p6,dddsd.lPitch);

     for y:= 1 to SCREEN_HEIGHT-2 do
     begin
          for x:= 1 to SCREEN_WIDTH-2 do
          begin
               //̐F̕ ɒ
               p6^:=FDivTable[p1^+p2^+p3^+p4^+p5^];

               Inc(p1);
               Inc(p2);
               Inc(p3);
               Inc(p4);
               Inc(p5);
               Inc(p6);
          end;
          Inc(p1,2);
          Inc(p2,2);
          Inc(p3,2);
          Inc(p4,2);
          Inc(p5,2);
          Inc(p6,2);
     end;
     FSurface.UnLock;
     FBlur.UnLock;
     FSurface.BltFast(FBlur,0,0,Rect(0,0,FSurface.Width,FSurface.Height),
                      DDBLTFAST_WAIT);
{$ELSE}
     FBlur.BltFast(FSurface,0,0,Rect(0,0,FSurface.Width,FSurface.Height),
                   DDBLTFAST_WAIT);
{$ENDIF}
end;

procedure TMainForm.Fps;
var
   t:Integer;
begin
     t:=GetTickCount;
     if t-FFpsTime > 1000 then
     begin
          FFps:=FFpsTemp;
          FFpsTemp:=0;
          FFpsTime:=t;
     end;

     Inc(FFpsTemp);
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     if Key = VK_ESCAPE then Application.Terminate;

     if Key = VK_DOWN then Dec(FSpeed);
     if Key = VK_UP   then Inc(FSpeed);

     if Key = VK_LEFT  then Inc(FAngle,8);
     if Key = VK_RIGHT then Dec(FAngle,8);

     if Key = VK_F1    then DD.BackBuffer.SaveBMP(ExePath+'snap.bmp');
     if FSpeed <=0 then FSpeed:=1;
     
end;

procedure TMainForm.InitOffsetTable;
var
   y:Integer;
   p:pByte;
   //pofy:ppByte;
   ddsd:DDSurfaceDesc2;
begin
     //GetMem(FOffSetTable,SCREEN_HEIGHT);
     //pofy:=FOffSetTable;

     FSurface.Lock(ddsd);
     p:=ddsd.lpSurface;
     for y:=0 to SCREEN_HEIGHT-1 do
     begin
          FOffsetTable[y]:=p;
          //pofy^:=p;
          Inc(p,ddsd.lPitch);
          //Inc(pofy);
     end;
     FSurface.UnLock;
end;

procedure TMainForm.InitTable;
var
   i:Integer;
   a:Integer;
begin
     FRndStep:=0;
     FRndIntStep:=0;
     FAngle  :=SIN_TERM shr 2;

     for i:= 0 to 256*5-1 do
     begin
          {$IFDEF USE_FIXEDDECIMAL}
          a:=Round(i / (GENSUI/65536));
          {$ELSE}
          a:=Round(i / GENSUI);
          {$ENDIF}
          if a > PARTICLE_COLOR then a:=PARTICLE_COLOR;
          FDivTable[i]:=a;
     end;

     for i:= 0 to RAND_SIZE-1 do
     begin
          FRandTable[i]:=Random;
     end;

     for i:= 0 to RAND_SIZE-1 do
     begin
          FRandIntTable[i]:=Random(PARTICLE_ENERGY*2);
     end;

     for i:= 0 to 256*2-1 do
     begin
          FAddTable[i]:=i;
          if i > PARTICLE_COLOR then
             FAddTable[i]:=PARTICLE_COLOR;
     end;
end;

function TMainForm.RandomFast: Single;
begin
     FRndStep:=(FRndStep+1) and RAND_SIZE_MASK;
     result:=FRandTable[FRndStep];
end;

procedure TMainForm.UpdateMouse;
var
   dx,dy:Integer;
begin
     DI.Scan(DI_MOUSE);

     Inc(FMouseX,DI.AStick.X);
     Inc(FMouseY,DI.AStick.Y);

     if FMouseX < 0 then FMouseX:=0;
     if FMouseX > SCREEN_WIDTH then FMouseX:=SCREEN_WIDTH;
     if FMouseY < 0 then FMouseY:=0;
     if FMouseY > SCREEN_HEIGHT then FMouseY:=SCREEN_HEIGHT;

     if FMouseDowned then
     begin
          FCenterX:=FMouseX;
          FCenterY:=FMouseY;
     end;

     dx:=  FMouseX-FCenterX;
     dy:=-(FMouseY-FCenterY);

     FStrong:=Round(Sqrt(dx*dx+dy*dy)*0.6);

     FAngle:=ATan2(dx,dy);
end;

procedure TMainForm.InitMouse;
begin
     FMouseX:=SCREEN_WIDTH  div 2;
     FMouseY:=SCREEN_HEIGHT div 2-50;
     FCenterX:=SCREEN_WIDTH  div 2;
     FCenterY:=SCREEN_HEIGHT div 2;
end;

function TMainForm.RandomIntFast: Integer;
begin
     FRndIntStep:=(FRndIntStep+1) and RAND_SIZE_MASK;
     result:=FRandIntTable[FRndIntStep];
end;

procedure TMainForm.LoadIni;
var
   IniFile:TIniFile;
begin
     IniFile:=TIniFile.Create(FIniFileName);
     with IniFile do
     begin
          SCREEN_WIDTH := ReadInteger('Screen','Width' ,640);
          SCREEN_HEIGHT:= ReadInteger('Screen','Height',480);

          MAX_PARTICLE    := ReadInteger('Particle','Max',1024);
          PARTICLE_COLOR  := ReadInteger('Particle','Color',254);
          PARTICLE_ENERGY := ReadInteger('Particle','Energy',2000);
          ANGLE_RANGE     := ReadInteger('Particle','AngleRange',256);
          //PARTICLE_SIZE   := ReadInteger('Particle','Size',1);

          {$IFDEF USE_FIXEDDECIMAL}
          M       := Round(ReadFloat('Particle','M',3)*65536);
          GRAVITY := Round(ReadFloat('Environment','Gravity',9.8)*65536);
          GENSUI  := Round(ReadFloat('Environment','Gensui',5.8)*65536);
          BOUND   := Round(ReadFloat('Environment','Bound',0.5)*65536);
          FV      := Round(ReadFloat('Environment','FV',100)*65536);
          {$ELSE}
          M       := ReadFloat  ('Particle','M',3);
          GRAVITY := ReadFloat('Environment','Gravity',9.8);
          GENSUI  := ReadFloat('Environment','Gensui',5.8);
          BOUND   := ReadFloat('Environment','Bound',0.5);
          FV      := ReadFloat('Environment','FV',100);
          {$ENDIF}
     end;
     IniFile.Free;
end;

procedure TMainForm.SaveIni;
var
   IniFile:TIniFile;
begin
     IniFile:=TIniFile.Create(FIniFileName);
     with IniFile do
     begin
          WriteInteger('Screen','Width' ,SCREEN_WIDTH);
          WriteInteger('Screen','Height',SCREEN_HEIGHT);

          WriteInteger('Particle','Max',MAX_PARTICLE);
          WriteInteger('Particle','Color',PARTICLE_COLOR);
          WriteInteger('Particle','Energy',PARTICLE_ENERGY);

          WriteFloat('Particle','AngleRange',ANGLE_RANGE);

          {$IFDEF USE_FIXEDDECIMAL}
          WriteFloat('Particle','M',M/65536);
          WriteFloat('Environment','Gravity',GRAVITY/65536);
          WriteFloat('Environment','Gensui',GENSUI/65536);
          WriteFloat('Environment','Bound',BOUND/65536);
          WriteFloat('Environment','FV',FV/65536);
          {$ELSE}
          WriteFloat('Particle','M',M);
          WriteFloat('Environment','Gravity',GRAVITY);
          WriteFloat('Environment','Gensui',GENSUI);
          WriteFloat('Environment','Bound',BOUND);
          WriteFloat('Environment','FV',FV);
          {$ENDIF}
     end;
     IniFile.Free;
end;

procedure TMainForm.FreeOffsetTable;
begin
     //FreeMem(FOffSetTable,SCREEN_HEIGHT);
end;

procedure TMainForm.FreeWater;
begin
     FreeMem(FWater,MAX_PARTICLE*SizeOf(TParticle));
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
     SaveIni;
     FreeWater;
     FreeOffSetTable;
end;

procedure TMainForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
     FMouseDowned:=Button = mbLeft;
end;

procedure TMainForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
     if Button = mbLeft then FMouseDowned:=False;
end;

end.
